function [logL, prior]=Lhood_BRT(par,kflag,nfarm,D,I)
%
% [logL, prior]=Lhood_BRT(par,kflag,nfarm,D,I)
%
% Matlab function to compute the log likelihood for the model for
% the geographic spread of Schmallenberg virus (at NUTS2 region level)
%
% Inputs:
% par - array containing model parameters
% kflag - flag indicating kernel to use:
%         1 - fat-tailed, density-dependent
%         2 - fat-tailed, density-independent
%         3 - Gaussian, density-dependent
%         4 - Gaussian, density-independent
%         5 - exponential, density-dependent
%         6 - exponential, density-independent
% nfarm - array containing number of farms in each region
% D - matrix containing distances between centroids of each region
% I - array indicating whether or not a region is infectious each day
%
% Outputs:
% logL - log likelihood for parameters
% prior - log(joint prior) for parameters

%==========================================================================
% PREPARE THE INPUTS
% Back-transform the parameters
h0=exp(par(1));
a=exp(par(2));
if kflag<=2,
    d0=exp(par(3));
end
%==========================================================================

%==========================================================================
% COMPUTE THE JOINT PRIOR
% Assume non-informative priors for all parameters
prior=log(gampdf(h0,1,100))+...
      log(gampdf(a,1,100));
if kflag<=2,
    prior=prior+log(gampdf(d0,1,100));
end
%==========================================================================

%==========================================================================
% COMPUTE THE LOG LIKELIHOOD
% Find the first and last times with infection
t0=find(sum(I,1)>0,1,'first');
t1=find(sum(I,1)>0,1,'last');

% Compute the relative vector activity
vseas=[-1.71; -1.56; -3.74; -1.49; -1.00];
va=exp((vseas(1)+vseas(2).*sin(2.0.*pi.*(t0:t1)./365.0)+...
                 vseas(3).*cos(2.0.*pi.*(t0:t1)./365.0)+...
                 vseas(4).*sin(4.0.*pi.*(t0:t1)./365.0)+...
                 vseas(5).*cos(4.0.*pi.*(t0:t1)./365.0)));
va=va./max(va);

% Initialise the force of infection
lambda=zeros(size(I,1),t1-t0+1);

% For each region ...
for r=1:size(I,1),
    x=[1:r-1 r+1:size(I,1)];

% Compute the kernel
    if kflag==1 || kflag==2,
	    k=1./(1+(D(x,r)./d0).^a);
    elseif kflag==3 || kflag==4,
	    k=exp(-(a.*D(x,r)).^2);
    elseif kflag==5 || kflag==6,
	    k=exp(-a.*D(x,r));
    end
        
% If the kernel is density independent ...
    if kflag==2 || kflag==4 || kflag==6,
        k=k./sum(k);
    end

% Compute the force of infection
    lambda(r,:)=h0.*(nfarm(r,1)+nfarm(r,2)).*...
                sum(I(x,t0:t1).*((k.*(nfarm(x,1)+nfarm(x,2)))*va),1);

end

% Evaluate the contribution to the log likelihood of regions which were
% not infected
u=(sum(I,2)==0);
logL=-sum(sum(lambda(u,:)));

% Add the contribution to the log likelihood for regions which were
% infected
i=find(sum(I,2)>0);
for j=1:length(i),
    
% Determine when the region became infected
    ti=find(I(i(j),:)>0,1,'first');
    
% If the region was not that infected initially ...
    if ti>t0,
        logL=logL-sum(lambda(i(j),1:ti-t0))+...
                  log(1-exp(-lambda(i(j),ti-t0+1)));

    end

end
%==========================================================================
