function [logL, prior]=Lhood_WRISero(par,tC,tS,NC,NS,IC,IS,C,S)
%
% [logL, prior]=Lhood_WRISero(par,tC,tS,NC,NS,C,S)
%
% Matlab function to compute the log likelihood for the number of holdings
% infected with Schmallenberg virus in each region (at NUTS2 region level)
% incorpating serosurveillance data for Belgium and the Netherlands
%
% Inputs:
% par - array containing model parameters
% nfarm - array containing number of farms in each region (col. 1 - cattle;
%         col. 2 - sheep)
% tC, tS - cumulative vector activity for each region
% NC, NS - number of cattle or sheep farms reporting cases for each region
% IC, IS - number of cattle or sheep farms positive (col. 1) and
%          sampled (col. 2) as part of serosurveillance
% C, S - number of cattle or sheep holdings in each region
%
% Outputs:
% logL - log likelihood for parameters
% prior - log(joint prior) for parameters

%==========================================================================
% PREPARE THE INPUTS
% Determine the number of regions reporting holdings with cases in cattle
% or sheep
ncattle=length(NC);
nsheep=length(NS);

% Extract and back-transform the parameters
lambdaC=exp(par(1:ncattle));
lambdaS=exp(par(ncattle+1:ncattle+nsheep));
rhoC=1./(1+exp(-par(ncattle+nsheep+1:2*ncattle+nsheep)));
rhoS=1./(1+exp(-par(2*ncattle+nsheep+1:2*(ncattle+nsheep))));
mu_lambdaC=exp(par(2*(ncattle+nsheep)+1));
k_lambdaC=exp(par(2*(ncattle+nsheep)+2));
mu_lambdaS=exp(par(2*(ncattle+nsheep)+3));
k_lambdaS=exp(par(2*(ncattle+nsheep)+4));
aC=exp(par(2*(ncattle+nsheep)+5));
bC=exp(par(2*(ncattle+nsheep)+6));
aS=exp(par(2*(ncattle+nsheep)+7));
bS=exp(par(2*(ncattle+nsheep)+8));
%==========================================================================

%==========================================================================
% COMPUTE THE JOINT PRIOR
% Assume hierarchical structure for the within-region forces of infection
prior=sum(log(gampdf(lambdaC,k_lambdaC,mu_lambdaC/k_lambdaC)))+...
      sum(log(gampdf(lambdaS,k_lambdaS,mu_lambdaS/k_lambdaS)));

% Assume hierarchical structure for the within-region probabilities of
% reporting
prior=prior+sum(log(betapdf(rhoC,aC,bC)))+...
            sum(log(betapdf(rhoS,aS,bS)));
  
% Assume non-informative priors for the hierarchical parameters
% prior=prior+log(gampdf(mu_lambdaC,1,100))+...
prior=prior+log(gampdf(mu_lambdaC,1,100))+...
            log(gampdf(k_lambdaC,1,100))+...
            log(gampdf(mu_lambdaS,1,100))+...
            log(gampdf(k_lambdaS,1,100))+...
            log(gampdf(aC,1,100))+...
            log(gampdf(bC,1,100))+...
            log(gampdf(aS,1,100))+...
            log(gampdf(bS,1,100));
%==========================================================================

%==========================================================================
% COMPUTE THE LOG LIKELIHOOD
% Compute the expected number of affected cattle and sheep holdings in each
% region
muC=lambdaC.*C.*tC;
muS=lambdaS.*S.*tS;

% Initialise the log likelighood
logL=0;

% For each region with data for cattle holdings ...
for j=1:ncattle,

% Set the range for the number of affected holdings
    y=max(NC(j),IC(j,1)):C(j);

% Compute the contribution to the log likelihood
    logL=logL+log(sum(binopdf(NC(j),y,rhoC(j)).*...
                      hygepdf(IC(j,1),C(j),y,IC(j,2)).*...
                      poisspdf(y,muC(j))));

end

% For each region with data for sheep holdings ...
for j=1:nsheep,

% Set the range for the number of affected holdings
    y=max(NS(j),IS(j,1)):S(j);

% Compute the contribution to the log likelihood
    logL=logL+log(sum(binopdf(NS(j),y,rhoS(j)).*...
                      hygepdf(IS(j,1),S(j),y,IS(j,2)).*...
                      poisspdf(y,muS(j))));

end
%==========================================================================
