function ParEstBRT(kflag,nchains,nsamp,nburnin,nthin)
%
% ParEstBRT(kflag,nchains,nsamp,nburnin,nthin)
%
% Matlab function to implement Bayesian inference of the transmission
% parameters for SBV based on a analysis of case data for Europe at NUTS2
% level
%
% Inputs:
% kflag - flag indicating kernel to use:
%         1 - fat-tailed, density-dependent
%         2 - fat-tailed, density-independent
%         3 - Gaussian, density-dependent
%         4 - Gaussian, density-independent
%         5 - exponential, density-dependent
%         6 - exponential, density-independent
% nchains - number of chains to run
% nsamp - number of samples to use when estimating parameters
% nburnin - number of samples to discard before estimating parameters
% nthin - number of samples by which to thin each chain
%
% Outputs: none (N.B. All chains are saved rather than provided as output
% arguments)

% Initialise the random number generator
stream=RandStream('mt19937ar','Seed',round(sum(1000*clock)));
RandStream.setGlobalStream(stream);

% Create cell arrays storing the chain
ParSamp=cell(1,nchains);

% For each chain ...
for chain=1:nchains,

%==========================================================================
% PREPARATORY STUFF
% Load the data
    varload=load(SBVInEurope_DataForBRTModel');

% Extract that which is needed for estimation (number of farms in each
% region, matrix of distance between centroids and number of infection
% status of the region at each time point)
    nfarm=varload.nfarm;
    D=varload.D;
    I=varload.I;

% Specify the number of parameters in each component
    if kflag<=2,
        npar=3;
    elseif kflag>=3,
        npar=2;
    end

% Set the initial scaling for the proposal distribution
    sf=(2.38.^2)./npar;

% Set the counter for the number of accepted samples
    n_accept=0;
%==========================================================================

%==========================================================================
% INITIALISE THE CHAIN
% Set the initial point for sampling
    display('Initialising chain')
    CurrL=NaN;
    prior=NaN;
    while isfinite(CurrL+prior)==0,

% Generate a parameter set
        if kflag<=2,
            par=log([unifrnd(0,1e-5); unifrnd(2,6); log(unifrnd(2,3))]);
        elseif kflag>=3,
            par=log([unifrnd(0,1e-5); unifrnd(0.001,0.01)]);
        end

% Compute the log-likelihood and joint prior
        [CurrL,prior]=Lhood_BRT(par,kflag,nfarm,D,I);

    end
%==========================================================================

%==========================================================================
% RUN THE CHAIN
% Create the array storing the chain
    ParSampC=zeros(nsamp/nthin,size(par,1)+2);
    iter=1;
    
% Sample parameter space
    display('Sampling parameter space')
    for samp=1:nsamp+nburnin,

% Indicate what's going on
        display(['Chain: ' num2str(chain) ', Sample: ' num2str(samp) ...
                 ', Acceptance: ' num2str(100*n_accept/samp,3) '%'])

% Generate the new parameter set
        if samp<=nburnin && samp<=2*npar,
            SIGp=0.01*eye(npar);
        elseif samp<=nburnin && samp>2*npar,
            SIGp=sf.*(SIG+0.01*eye(npar));
        end
        par_new=par+mvnrnd(zeros(1,npar),SIGp)';

% Generate the new parameter set and compute the log likelihood and prior
        [NewL,prior_new]=Lhood_BRT(par_new,kflag,nfarm,D,I);

% Test whether of not to accept the new parameter set
        u=unifrnd(0,1);
        if isnan(NewL+prior_new)==0 && ...
           u<min(1,exp((NewL+prior_new)-(CurrL+prior))),

% If it is accepted, update the chain
            par=par_new;
            CurrL=NewL;
            prior=prior_new;
            n_accept=n_accept+1;

% Update the covariance matrix for the proposal distribution
            if samp<=nburnin && n_accept==1,
                pbar=mean([par par_new],2);
                SIG=cov([par'; par_new']);
            elseif samp<=nburnin && n_accept>1,
                pbar_prev=pbar;
                pbar=(n_accept./(n_accept+1)).*pbar_prev+...
                     (1./(n_accept+1)).*par_new;
                SIG=((n_accept-1)./n_accept).*SIG+...
                    (1./n_accept).*(n_accept.*(pbar_prev*pbar_prev')-...
                                    (n_accept+1).*(pbar*pbar')+...
                                    (par_new*par_new'));
            end

        end

% Every one hundred samples during burn-in, tune the scaling for the
% proposal distribution to ensure an acceptance rate of 20-40%
        if samp<=nburnin && mod(samp,100)==1 && n_accept/samp<0.2,
            sf=sf/2;
        elseif samp<=nburnin && mod(samp,100)==1 && n_accept/samp>0.4,
            sf=2*sf;
        end

% Save iterations of the chain, thinning as specified
        if nthin==1,
            ParSampC(samp,:)=[par' prior CurrL];
        elseif samp>nburnin && mod(samp,nthin)==1,
            ParSampC(iter,:)=[par' prior CurrL];
            iter=iter+1;
        end
        
    end

% Store the chain output
    ParSamp{chain}=ParSampC;
%==========================================================================
    
end

%==========================================================================
% COMPUTE DIC AND pD
% Compute the deviance for each sample
Dev=[];
PS=[];
for chain=1:nchains,
    Dev=[Dev; -2*ParSamp{chain}(:,end)];
    PS=[PS; ParSamp{chain}(:,1:end-2)];
end

% Compute the mean deviance
Dbar=mean(Dev);

% Compute the deviance at the posterior mean for the parameters
Dhat=-2*Lhood_BRT(mean(PS,1)',kflag,nfarm,D,I);

% Compute the DIC
DIC=2*Dbar-Dhat;

% Compute the effective number of parameters
pD=Dbar-Dhat;
%==========================================================================

% Construct the filename for the model
fname=['ParameterMCMCSamples_BRTKernel' num2str(kflag)];

% Save the results
save(fname,'ParSamp','nsamp','nburnin','DIC','pD')
