function ParEstWRI(nchains,nsamp,nburnin,nthin)
%
% ParEstWRI(nchains,nsamp,nburnin,nthin)
%
% Matlab function to implement Bayesian inference of the within-region
% incidence of SBV based on a analysis of case data for Europe at NUTS2
% level
%
% Inputs:
% nchains - number of chains to run
% nsamp - number of samples to use when estimating parameters
% nburnin - number of samples to discard before estimating parameters
% nthin - number of samples by which to thin each chain
%
% Outputs: none (N.B. All chains are saved rather than provided as output
% arguments)

% Initialise the random number generator
stream=RandStream('mt19937ar','Seed',round(sum(1000*clock)));
RandStream.setGlobalStream(stream);
    
%==========================================================================
% DATA ON REPORTED HOLDINGS
% Load the data. These are in the format of cell arrays containing:
% tC, tS - times with reported cases for each region
% NC, NS - number of cattle or sheep farms reporting cases for each region
varload=load('SBVInEurope_WRISeroData');
tC=varload.tC;
tS=varload.tS;
NC=varload.NC;
NS=varload.NS;

% Compute the seasonal vector activity
vseas=[-1.71; -1.56; -3.74; -1.49; -1.00];
va=exp((vseas(1)+vseas(2).*sin(2.0.*pi.*(1:730)./365.0)+...
                 vseas(3).*cos(2.0.*pi.*(1:730)./365.0)+...
                 vseas(4).*sin(4.0.*pi.*(1:730)./365.0)+...
                 vseas(5).*cos(4.0.*pi.*(1:730)./365.0)));
va=va./max(va);

% Compute the cumulative activity for each region
tC2=zeros(ncattle,1);
for j=1:ncattle,
    tC2(j)=sum(va(tC{j}(2):tC{j}(end)));
end
tC=tC2;
tS2=zeros(nsheep,1);
for j=1:nsheep,
    tS2(j)=sum(va(tS{j}(2):tS{j}(end)));
end
tS=tS2;
clear('tC2','tS2')
%==========================================================================

% Create cell arrays storing the chain
ParSamp=cell(1,nchains);

% For each chain ...
parfor chain=1:nchains,

%==========================================================================
% PREPARATORY STUFF
% Determine the number of parameters
    npar=nsheep+ncattle+4;

% Set the initial scaling for the proposal distribution
    sf=(2.38.^2)./npar;
    SIG=eye(npar);

% Set the counter for the number of accepted samples
    n_accept=0;
%==========================================================================

%==========================================================================
% INITIALISE THE CHAIN
% Set the initial point for sampling
    display('Initialising chain')
    CurrL=NaN;
    prior=NaN;
    while isfinite(CurrL+prior)==0,

% Generate forces of infection for each region
        lambdaC=gamrnd(2,NC./(2.*C.*tC));
        lambdaS=gamrnd(2,NS./(2.*S.*tS));

% Generate the hierarchical parameters
        p=gamfit(lambdaC);
        mu_lambdaC=p(1)*p(2);
        k_lambdaC=p(1);
        p=gamfit(lambdaS);
        mu_lambdaS=p(1)*p(2);
        k_lambdaS=p(1);

% Transform the parameters
        par=[log(lambdaC); log(lambdaS);
             log(mu_lambdaC); log(k_lambdaC);
             log(mu_lambdaS); log(k_lambdaS)];

% Compute the log-likelihood and joint prior
        [CurrL,prior]=Lhood_WRI(par,tC,tS,NC,NS,C,S);

    end
%==========================================================================

%==========================================================================
% RUN THE CHAIN
% Create the array storing the chain
    ParSampC=zeros(nsamp/nthin,size(par,1)+2);
    iter=1;
    
% Sample parameter space
    display('Sampling parameter space')
    for samp=1:nsamp+nburnin,

% Indicate what's going on
        display(['Chain: ' num2str(chain) ', Sample: ' num2str(samp) ...
                 ', Acceptance: ' num2str(100*n_accept/samp,3) '%'])

% Generate the new parameter set
        if samp<=nburnin && samp<=2*npar,
            SIGp=0.01*eye(npar);
        elseif samp<=nburnin && samp>2*npar,
            SIGp=sf.*(SIG+0.01*eye(npar));
        end
        par_new=par+mvnrnd(zeros(1,npar),SIGp)';

% Generate the new parameter set and compute the log likelihood and prior
        [NewL,prior_new]=Lhood_WRI(par_new,tC,tS,NC,NS,C,S);

% Test whether of not to accept the new parameter set
        u=unifrnd(0,1);
        if isnan(NewL+prior_new)==0 && ...
           u<min(1,exp((NewL+prior_new)-(CurrL+prior))),

% If it is accepted, update the chain
            par=par_new;
            CurrL=NewL;
            prior=prior_new;
            n_accept=n_accept+1;

% Update the covariance matrix for the proposal distribution
            if samp<=nburnin && n_accept==1,
                pbar=mean([par par_new],2);
                SIG=cov([par'; par_new']);
            elseif samp<=nburnin && n_accept>1,
                pbar_prev=pbar;
                pbar=(n_accept./(n_accept+1)).*pbar_prev+...
                     (1./(n_accept+1)).*par_new;
                SIG=((n_accept-1)./n_accept).*SIG+...
                    (1./n_accept).*(n_accept.*(pbar_prev*pbar_prev')-...
                                    (n_accept+1).*(pbar*pbar')+...
                                    (par_new*par_new'));
            end

        end

% Every one hundred samples during burn-in, tune the scaling for the
% proposal distribution to ensure an acceptance rate of 20-40%
        if samp<=nburnin && mod(samp,100)==1 && n_accept/samp<0.2,
            sf=sf/2;
        elseif samp<=nburnin && mod(samp,100)==1 && n_accept/samp>0.4,
            sf=2*sf;
        end

% Save iterations of the chain, thinning as specified
        if nthin==1,
            ParSampC(samp,:)=[par' prior CurrL];
        elseif samp>nburnin && mod(samp,nthin)==1,
            ParSampC(iter,:)=[par' prior CurrL];
            iter=iter+1;
        end
        
    end

% Store the chain output
    ParSamp{chain}=ParSampC;
%==========================================================================
    
end

%==========================================================================
% COMPUTE DIC AND pD
% Compute the mean deviance and posterior mean
Dev=[];
PS=[];
for chain=1:nchains,
    Dev=[Dev; -2*ParSamp{chain}(:,end)];
    PS=[PS; ParSamp{chain}(:,1:end-2)];
end
Dbar=mean(Dev);
pbar=mean(PS,1)';

% Compute the deviance at the posterior mean for the parameters
Dhat=-2*Lhood_WRI(pbar,tC,tS,NC,NS,C,S);

% Compute the DIC
DIC=2*Dbar-Dhat;

% Compute the effective number of parameters
pD=Dbar-Dhat;
%==========================================================================

%==========================================================================
% SAVE THE CHAINS
% Construct the filename for the model
fname='ParameterMCMCSamples_WRI';

% Save the results
save(fname,'ParSamp','nsamp','nburnin','DIC','pD')
%==========================================================================
