function I=SimulateBRT(t0,tstop,I0,kflag,nfarm,D,h0,a,d0)
%
% I=SimulateBRT(t0,tstop,I0,kflag,nfarm,D,h0,a,d0)
%
% Matlab function to simulate the model for the geographic spread of
% Schmallenberg virus (at NUTS2 region level) for purposes of model
% checking
%
% Inputs:
% t0 - day of the incursion
% tstop - last day for the simulation (relative to 1 January)
% I0 - initial status of each region
% kflag - flag indicating kernel to use
% nfarm - array containing number of farms in each region
% D - matrix containing distances between centroids of each region
% h0, a, d0 - parameters for transmission between regions
%
% Output:
% I - array indicating whether or not a region is infectious each day

% Create the output arrays
I=zeros(size(nfarm,1),tstop);

% Compute the relative vector activity
vseas=[-1.71; -1.56; -3.74; -1.49; -1.00];
va=exp((vseas(1)+vseas(2).*sin(2.0.*pi.*(1:tstop)./365.0)+...
                 vseas(3).*cos(2.0.*pi.*(1:tstop)./365.0)+...
                 vseas(4).*sin(4.0.*pi.*(1:tstop)./365.0)+...
                 vseas(5).*cos(4.0.*pi.*(1:tstop)./365.0)));
va=va./max(va);

% Extract the initially-infected regions
i0=find(I0>0);

% Set their status in the infection array
for j=1:length(i0),
    I(i0(j),t0:tstop)=1;
end

% For each subsequent day ...
for day=t0+1:tstop,

% Initialise the force of infection
    lambda=zeros(size(I,1),1);
    
% For each region ...
    for r=1:size(I,1),
        x=[1:r-1 r+1:size(I,1)];

% Compute the kernel
        if kflag==1 || kflag==2,
            k=1./(1+(D(x,r)./d0).^a);
        elseif kflag==3 || kflag==4,
            k=exp(-(a.*D(x,r)).^2);
        elseif kflag==5 || kflag==6,
            k=exp(-a.*D(x,r));
        end
        
% If the kernel is density independent ...
        if kflag==2 || kflag==4 || kflag==6,
            k=k./sum(k);
        end

% Compute the force of infection
        lambda(r)=h0.*(nfarm(r,1)+nfarm(r,2)).*va(day).*...
                  sum(I(x,day-1).*k.*(nfarm(x,1)+nfarm(x,2)),1);

    end

% Test which regions become infected
    u=unifrnd(0,1,length(lambda),1);
    I(u<1-exp(-lambda) & I(:,day-1)==0,day)=1;

% For each region which becomes infected, set its status
    for r=1:size(I,1),
        if I(r,day-1)==0 && I(r,day)==1,
            I(r,day:tstop)=1;
        end
    end
    
end
